define('confluence-collaborative-editor-plugin/btf/util/option-selector', [], function () {
    /**
     * Selects an option from the given options based on the parameters given.
     * @param {Array} options This is an array of the form:
     *                             [ {
     *                                  option: {*},
     *                                  conditions: {
     *                                      conditionKey: <conditionValue> | <Array<conditionValue>>
     *                                  }
     *                               },
     *                               ...
     *                             ]
     *
     *                         The option field will be selected if the conditions are met given
     *                         the parameters.
     *
     *                         If the conditionKey value is a non-array then there must exist a parameter in the
     *                         parameters object with the same key and value.
     *
     *                         If the conditionKey value is an array then there must exist a parameter in the
     *                         parameters object with the same key where the value is found in the conditions
     *                         array. This is treated as an OR.
     *
     *                         It will select the first item from the options that matches all the conditions
     * @param {object} parameters   An object of the parameters to match against.
     * @param {*} defaultValue  Optional. A default value to return if none of the options match.
     * @returns {*} It returns the .option field of the first option object in the options @param whose conditions
     *              match the given parameters, or the default value if nothing matches.
     */
    return function selectOption(options, parameters, defaultValue) {
        // Loop through all the options first
        option_loop: for (var i = 0; i < options.length; i++) {
            var option = options[i];
            // Then check the conditions of each option
            condition_loop : for (var key in option.conditions) {
                if (option.conditions.hasOwnProperty(key)) {
                    // If the supplied parameters doesn't contain that key, then we move on
                    if (typeof parameters[key] === 'undefined') {
                        continue option_loop;
                    }
                    // Otherwise we need to check if
                    // 1. It's an Array
                    if (Array.isArray(option.conditions[key])) {
                        var test = parameters[key];
                        // If it's an array, it's treated as an OR so we just need to
                        // find one value from the array that matches the supplied parameter
                        for (var j = 0; j < option.conditions[key].length; j++) {
                            if (test === option.conditions[key][j]) {
                                // If we find one then we just move on to the next condition to check
                                continue condition_loop;
                            }
                        }
                        // If we don't find any, then we can't choose this option, so we move on
                        continue option_loop;
                        // or 2. It's not
                    } else if (option.conditions[key] !== parameters[key]) {
                        // If it's not an array and the value for the option doesn't match
                        // what is provided in the parameters, then we move on
                        continue option_loop;
                    }
                }
            }
            // If we haven't moved on, then it means all the conditions have passed with the parameters
            // so we choose this option to return
            return option.option;
        }

        return defaultValue;
    }
});