define('confluence-collaborative-editor-plugin/synchrony-handlers', [
    'jquery',
    'ajs',
    'confluence/meta',
    'confluence/legacy',
    'confluence-editor/editor/page-editor-message',
    'confluence-collaborative-editor-plugin/lib/SyncManager',
    'confluence-collaborative-editor-plugin/synchrony-util',
    'confluence-collaborative-editor-plugin/synchrony-content',
    'confluence-collaborative-editor-plugin/synchrony-presence'
], function (
    $,
    AJS,
    Meta,
    Confluence,
    Message,
    SyncManager,
    Util,
    Content,
    SynchronyPresence
) {
    'use strict';

    var lastKnownContributors = [];
    var $revField, statusIndicator, editor, htmlDoc, entity, syncManager;

    function handle(_entity, _editor, _statusIndicator) {
        Util.time("confluence.editor.synchrony.connect");
        entity = _entity;
        editor = _editor;
        statusIndicator = _statusIndicator;

        $revField = $('#syncRev');
        htmlDoc = editor.getBody();

        entity.on('init', handleInit)
            .on('update', handleUpdate)
            .on('ack', handleAck)
            .on('connected', handleConnected)
            .on('disconnected', handleDisconnected)
            .on('error', handleError);

        // responsible for managing entity.start/entity.stop abstraction layer
        syncManager = new SyncManager(entity, {
            DEBUG_MODE: true
        });

        Content.bindPostPasteFix();
        Content.readTitleFromRootElement(htmlDoc);
        Confluence.Editor.overrideBeforeSave(handleBeforeSave);
        $('#content-title').keyup(Content.writeTitleToRootElement);
    }

    function handleHeartbeatEvent(e, data) {
        if (data.contributors) {
            lastKnownContributors = data.contributors;
            AJS.unbind(e, handleHeartbeatEvent);
        }
    }

    AJS.bind('editor-heartbeat', handleHeartbeatEvent);

    function handleInit(event) {
        $revField.val(event.rev.toString());
    }

    function handleAck(event) {
        $revField.val(event.rev.toString());

        AJS.trigger('synchrony.entity.ack', { 'pendingChanges': event.pending.length > 0 });
    }

    function handleDisconnected() {
        statusIndicator.onDisconnectedState();
        AJS.log('Synchrony disconnected!');
    }

    function handleError(event) {
        //TODO: We probably want to fire a confluence.synchrony.connection.error event similar to the backend to indicate a connection failure here, but the error comes from within sockjs and we can't handle it yet. https://jira.atlassian.com/browse/WD-463
        if (event.level === 'fatal') {
            AJS.trigger("editor.error.message", {
                disablePublish: true
            });
        }
    }

    function handleConnected(event) {
        Util.timeEnd("confluence.editor.synchrony.connect");
        AJS.log('Synchrony connected.');

        SynchronyPresence.appendTo(event.sid, entity, Confluence.getContentId(), $('#rte-toolbar'), lastKnownContributors);
        statusIndicator.onConnectedState(entity);

        AJS.trigger('synchrony.connected');

        Content.fixTinymceCaretContainer(htmlDoc, editor);
    }

    function handleBeforeSave(e) {
        var timeout = 5000;
        var deferred = $.Deferred();

        function synchronySuccess() {
            deferred.resolve(e);
        }

        function onAck(event) {
            if (!event.pending.length) {
                entity.off('ack', onAck);
                $revField.val(event.rev.toString());
                synchronySuccess();
            }
        }

        var synced = !entity.ackState().pending.length;
        if (synced) {
            synchronySuccess();
        }
        else {
            entity.on('ack', onAck);

            setTimeout(function () {
                entity.off('ack', onAck);
                deferred.reject({
                    origin: 'synchrony',
                    cause: 'timeout',
                    message: timeout / 1000 + 's timeout exceeded'
                });
            }, timeout);
        }

        return deferred.promise();
    }

    function userNameFromUpdateEvent(event) {
        var latestWithUser = Util.getLatestRevisionWithAttr(event.revisions, "user");
        return latestWithUser ? latestWithUser.meta.user : AJS.I18n.getText("anonymous.name");
    }

    function notifyPublished(userName) {
        Message.handleMessage('editor.synchrony.page-published', {
            type: 'info',
            message: AJS.I18n.getText('editor.synchrony.publish-page.message', userName),
            close: 'auto'
        });

        AJS.trigger('analyticsEvent', {
            name: 'confluence.synchrony.external-changes.publish'
        });
        AJS.trigger('editor-shared-drafts-published');
    }

    function updateMetadataForPublish() {
        Meta.set("new-page", false);
        Meta.set("page-id", Confluence.getContentId());
        Meta.set("draft-id", "0"); // We don't know the draft ID at this stage, but we know it's invalid.

        // Here we enable the discard button only if limited mode is not enabled.
        if (!Confluence.Editor.isLimitedModeEnabled()) {
            Confluence.Editor.UI.setButtonState(true, $('#rte-button-discard'));
        }
    }

    function handleUpdate(event) {
        if (event.updateType === 'remote') {
            $revField.val(event.revisions[event.revisions.length - 1].rev.toString());

            if (Util.hasRevisionTrigger(event.revisions, 'reset')) {
                Message.handleMessage('editor.synchrony.revert-page', {
                    type: 'info',
                    message: AJS.I18n.getText('editor.synchrony.revert-page.message', userNameFromUpdateEvent(event))
                });

                // Collaborators will be cleared from the draft upon reset. If the client continues editing, they will
                // need to be added as a collaborator again
                Meta.set('has-collaborated', false);

                resetDirtyFlag();

                AJS.trigger('analyticsEvent', {
                    name: 'confluence.synchrony.external-changes.revert'
                });

                //CONFDEV-47416: notify other modules to cleanup data if necessary
                AJS.trigger("editor-shared-drafts-discarded");

                // Here we trigger a heartbeat to keep things in sync.
                Confluence.Editor.heartbeat();
            }

            if (Util.hasRevisionTrigger(event.revisions, 'publish')) {
                if (Content.isUnpublished()) {
                    updateMetadataForPublish();
                }
                // Collaborators will be cleared from the draft upon publishing. If the client continues editing, they will
                // need to be added as a collaborator again
                Meta.set('has-collaborated', false);

                resetDirtyFlag();

                notifyPublished(userNameFromUpdateEvent(event));

                // Here we trigger a heartbeat to keep things in sync.
                Confluence.Editor.heartbeat();
            }

            if (Util.hasRevisionType(event.revisions, 'external')) {
                AJS.trigger('editor.external.change');
            }

            var latestWithConfVersion = Util.getLatestRevisionWithAttr(event.revisions, "confVersion");

            if (latestWithConfVersion && latestWithConfVersion.meta.confVersion) {
                $('meta[name="page-version"]').attr('content', latestWithConfVersion.meta.confVersion);
                $('meta[name="ajs-page-version"]').attr('content', latestWithConfVersion.meta.confVersion);
                Meta.set('page-version', latestWithConfVersion.meta.confVersion);
            }
        }
        if (event.updateType === 'local') {
            /**
             * when user types a letter (local change), no undo level is added to undo stack
             * => editor is not marked as dirty
             * so, should mark editor as dirty on local change
             */
            editor.setDirty(true);
        }
        // DOM mutations must not happen in the update event
        setTimeout(function () {
            if (event.updateType === 'init' || event.updateType === 'remote') {
                //We consider init a remote change because of the content reconciliation,
                //which now uses editor format directly (not modified by editor javascript)
                //pagelayouts listen for this and otherwise they wouldn't be editable
                AJS.trigger('editor.remote.change'); //CONFDEV-43345
                Content.readTitleFromRootElement(htmlDoc);
            }

            if (event.updateType == 'init') {
                //fix for https://jira.atlassian.com/browse/CONFSRVDEV-6580 (and related https://jira.atlassian.com/browse/CONFDEV-40982)
                editor.undoManager.ignore(function(){
                    _fixMentions(htmlDoc);
                });
            }

            Content.fixTinymceCaretContainer(htmlDoc, editor);

            if (event.updateType === 'local') {
                /**
                 * If the undo manager discards changes (> 256 changes)
                 * and the user undoes all >256 changes we will incorrectly
                 * call reset and not show the discard dialog even though
                 * the user has undoable changes that have been discarded
                 */
                if (!editor.undoManager.hasUndo()) {
                    editor.setDirty(false);
                }
                AJS.trigger('editor.local.change');
            }
        }, 0);
    }

    function resetDirtyFlag() {
        /**
         * edge case might happen when user B discard/publish
         * before user A get notification, he keeps typing
         * when user A get notification, dirty flag MUST NOT be reset
         * because new letters haven't been saved yet
         * => only reset dirty flag when there is no pending change
         */
        if (entity.ackState().pending.length === 0) {
            // We need to set the dirty flag back to false when unpublished changes are discarded/published
            editor.setDirty(false);
        }
    }

    function _fixMentions(htmlDoc) {
            var userMentionSelector = "a[data-linked-resource-id][data-linked-resource-type='userinfo'][href]";
            var mentionsList = htmlDoc.querySelectorAll(userMentionSelector);
            mentionsList.forEach(function (it) {
                it.removeAttribute("data-linked-resource-id");
                //fix up bug where we lose the link if the username attribute isn't set (since we are going to be bypassing the resource-id codepath on the serverside)
                var username = it.getAttribute("data-username");
                if (!username || username.trim().length == 0) {
                    //guess by using the username in the link's url
                    var href = it.getAttribute("href");
                    username = href.split("~")[1];
                }
                it.setAttribute("username" , username);
            });
            return mentionsList.length;
    }

    return {
        handle: handle
    };
});
