define('confluence-collaborative-editor-plugin/synchrony-util', [
    'window',
    'jquery',
    'underscore',
    'confluence/meta',
    'confluence/legacy'
], function (
    global,
    $,
    _,
    Meta,
    Confluence
) {
    'use strict';

    function retrieveMetadata(name) {
        // First we check if the AJS.Meta produces a result.
        var metaValue = Meta.get(name);
        if (metaValue === undefined) {
            // If not we will try reading the value from the DOM.
            metaValue = $('meta[name="ajs-' + name + '"]').attr('content');
            if (metaValue !== undefined) {
                // Here we set the AJS.Meta value as a contingency (also doubles as a cache for this call).
                Meta.set(name, metaValue);
            } else {
                console.error("Missing '" + name + "' metadata value");
            }
        }
        return metaValue;
    }

    function getEntityId() {
        return '/' + retrieveMetadata('synchrony-app-id') + '/confluence-' + Confluence.getContentId();
    }

    function isContentTypeSupported() {
        var contentType = retrieveMetadata('content-type');
        return contentType == 'page' || contentType == 'blogpost';
    }

    function entityExists() {
        return Confluence.getContentId() !== '0';
    }

    function isSynchronyEnabled() {
        return retrieveMetadata('synchrony-dark-enabled') === true;
    }

    function isSynchronyEnabledAndSupported() {
        return isSynchronyEnabled() && isContentTypeSupported() && entityExists();
    }

    function getServiceUrl() {
        return retrieveMetadata('synchrony-service-uri');
    }

    function getXhrFallbackFlag() {
        return retrieveMetadata('use-xhr-fallback');
    }

    function getResourceUrl(path) {
        return retrieveMetadata('synchrony-resources-uri') + path;
    }

    function getLatestRevisionWithAttr(revisions, attr) {
        return _.last(revisions.filter(function(revision){
            return revision.meta && revision.meta[attr];
        }));
    }

    function hasRevisionType(revisions, type) {
        return revisions.some(function (revision) {
            return revision.meta && revision.meta.type === type;
        });
    }

    function hasRevisionTrigger(revisions, trigger) {
        return revisions.some(function (revision) {
            return revision.meta && revision.meta.trigger === trigger;
        });
    }

    function asArray() {
        return Array.prototype.slice.call(arguments);
    }

    function time(key) {
        Confluence.debugTime && Confluence.debugTime(key);
    }

    function timeEnd(key) {
        Confluence.debugTimeEnd && Confluence.debugTimeEnd(key);
    }

    function loadScript(path) {
        var deferred = $.Deferred();
        var head = global.document.getElementsByTagName('head')[0];
        var script = global.document.createElement('script');
        script.onload = deferred.resolve;
        script.onerror = deferred.reject;
        script.type = 'text/javascript';
        script.src = getResourceUrl(path);
        head.appendChild(script);
        return deferred.promise();
    }

    return {
        retrieveMetadata: retrieveMetadata,
        getEntityId: getEntityId,
        isSynchronyEnabledAndSupported: isSynchronyEnabledAndSupported,
        getServiceUrl: getServiceUrl,
        getXhrFallbackFlag: getXhrFallbackFlag,
        getLatestRevisionWithAttr: getLatestRevisionWithAttr,
        hasRevisionType: hasRevisionType,
        hasRevisionTrigger: hasRevisionTrigger,
        asArray: asArray,
        time: time,
        timeEnd: timeEnd,
        loadScript: loadScript
    };
});
